@@ -4,6 +4,7 @@ from __future__ import print_function
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
+import re
 import six
 import gzip
 import zipfile
@@ -21,64 +22,118 @@ from . import config
 # Module API
 
 class Stream(object):
-    '''Stream of tabular data.
+    """Stream of tabular data.
 
     This is the main `tabulator` class. It loads a data source, and allows you
     to stream its parsed contents.
 
-    Args:
-        source (str): Path to file as ``<scheme>://path/to/file.<format>``. If
-            not explicitly set, the scheme (file, http, ...) and format (csv, xls,
-            ...) are inferred from the source string.
-        headers (Union[int, List[int], List[str]], optional): Either a row
+    # Arguments
+
+        source (str):
+            Path to file as ``<scheme>\\://path/to/file.<format>``.
+            If not explicitly set, the scheme (file, http, ...) and
+            format (csv, xls, ...) are inferred from the source string.
+
+        headers (Union[int, List[int], List[str]], optional):
+            Either a row
             number or list of row numbers (in case of multi-line headers) to be
             considered as headers (rows start counting at 1), or the actual
             headers defined a list of strings. If not set, all rows will be
             treated as containing values.
-        scheme (str, optional): Scheme for loading the file (file, http, ...).
-             If not set, it'll be inferred from `source`.
-        format (str, optional): File source's format (csv, xls, ...). If not
-             set, it'll be inferred from `source`. inferred
-        encoding (str, optional): Source encoding. If not set, it'll be inferred.
-        compression (str, optional): Source file compression (zip, ...). If not
-            set, it'll be inferred.
-        allow_html (bool, optional): Allow the file source to be an HTML page.
+
+        scheme (str, optional):
+            Scheme for loading the file (file, http, ...).
+            If not set, it'll be inferred from `source`.
+
+        format (str, optional):
+            File source's format (csv, xls, ...). If not
+            set, it'll be inferred from `source`. inferred
+
+        encoding (str, optional):
+            Source encoding. If not set, it'll be inferred.
+
+        compression (str, optional):
+            Source file compression (zip, ...). If not set, it'll be inferred.
+
+        allow_html (bool, optional):
+            Allow the file source to be an HTML page.
             If False, raises ``exceptions.FormatError`` if the loaded file is
             an HTML page. Defaults to False.
-        sample_size (int, optional): Controls the number of sample rows used to
+
+        sample_size (int, optional):
+            Controls the number of sample rows used to
             infer properties from the data (headers, encoding, etc.). Set to
             ``0`` to disable sampling, in which case nothing will be inferred
             from the data. Defaults to ``config.DEFAULT_SAMPLE_SIZE``.
-        bytes_sample_size (int, optional): Same as `sample_size`, but instead
+
+        bytes_sample_size (int, optional):
+            Same as `sample_size`, but instead
             of number of rows, controls number of bytes. Defaults to
             ``config.DEFAULT_BYTES_SAMPLE_SIZE``.
-        ignore_blank_headers (bool, optional): When True, ignores all columns
+
+        ignore_blank_headers (bool, optional):
+            When True, ignores all columns
             that have blank headers. Defaults to False.
-        force_strings (bool, optional): When True, casts all data to strings.
+
+        ignore_listed_headers (List[str], optional):
+            When passed, ignores all columns with headers
+            that the given list includes
+
+        ignore_not_listed_headers (List[str], optional):
+            When passed, ignores all columns with headers
+            that the given list DOES NOT include
+
+        multiline_headers_joiner (str, optional):
+            When passed, it's used to join multiline headers
+            as `<passed-value>.join(header1_1, header1_2)`
+            Defaults to ' ' (space).
+
+        force_strings (bool, optional):
+            When True, casts all data to strings.
             Defaults to False.
-        force_parse (bool, optional): When True, don't raise exceptions when
+
+        force_parse (bool, optional):
+            When True, don't raise exceptions when
             parsing malformed rows, simply returning an empty value. Defaults
             to False.
-        skip_rows (List[Union[int, str]], optional): List of row numbers and
-            strings to skip. If a string, it'll skip rows that begin with it
-            (e.g. '#' and '//').
-        post_parse (List[function], optional): List of generator functions that
+
+        skip_rows (List[Union[int, str, dict]], optional):
+            List of row numbers, strings and regex patterns as dicts to skip.
+            If a string, it'll skip rows that their first cells begin with it e.g. '#' and '//'.
+            To skip only completely blank rows use `{'type'\\: 'preset', 'value'\\: 'blank'}`
+            To provide a regex pattern use  `{'type'\\: 'regex', 'value'\\: '^#'}`
+            For example\\: `skip_rows=[1, '# comment', {'type'\\: 'regex', 'value'\\: '^# (regex|comment)'}]`
+
+        skip_columns (str[]):
+            Alias for `ignore_listed_headers`.
+            If it contains an empty string it will activate `ignore_blank_headers`
+
+        pick_columns (str[]):
+            Alias for `ignore_not_listed_headers`.
+
+        post_parse (List[function], optional):
+            List of generator functions that
             receives a list of rows and headers, processes them, and yields
             them (or not). Useful to pre-process the data. Defaults to None.
-        custom_loaders (dict, optional): Dictionary with keys as scheme names,
+
+        custom_loaders (dict, optional):
+            Dictionary with keys as scheme names,
             and values as their respective ``Loader`` class implementations.
             Defaults to None.
-        custom_parsers (dict, optional): Dictionary with keys as format names,
+
+        custom_parsers (dict, optional):
+            Dictionary with keys as format names,
             and values as their respective ``Parser`` class implementations.
             Defaults to None.
-        custom_loaders (dict, optional): Dictionary with keys as writer format
+
+        custom_loaders (dict, optional):
+            Dictionary with keys as writer format
             names, and values as their respective ``Writer`` class
             implementations. Defaults to None.
+
         **options (Any, optional): Extra options passed to the loaders and parsers.
 
-    Returns:
-        Stream: The Stream instance.
-    '''
+    """
 
     # Public
 
@@ -93,9 +148,14 @@ class Stream(object):
                  sample_size=config.DEFAULT_SAMPLE_SIZE,
                  bytes_sample_size=config.DEFAULT_BYTES_SAMPLE_SIZE,
                  ignore_blank_headers=False,
+                 ignore_listed_headers=None,
+                 ignore_not_listed_headers=None,
+                 multiline_headers_joiner=' ',
                  force_strings=False,
                  force_parse=False,
                  skip_rows=[],
+                 skip_columns=None,
+                 pick_columns=None,
                  post_parse=[],
                  custom_loaders={},
                  custom_parsers={},
@@ -118,12 +178,29 @@ class Stream(object):
             else:
                 self.__headers = list(headers)
 
+        # Translate aliases
+        if pick_columns is not None:
+            ignore_not_listed_headers = pick_columns
+        if skip_columns is not None:
+            ignore_listed_headers = skip_columns
+            if '' in skip_columns:
+                ignore_blank_headers = True
+
         # Set skip rows
         self.__skip_rows_by_numbers = []
+        self.__skip_rows_by_patterns = []
         self.__skip_rows_by_comments = []
+        self.__skip_rows_by_presets = {}
         for directive in copy(skip_rows):
             if isinstance(directive, int):
                 self.__skip_rows_by_numbers.append(directive)
+            elif isinstance(directive, dict):
+                if directive['type'] == 'regex':
+                    self.__skip_rows_by_patterns.append(re.compile(directive['value']))
+                elif directive['type'] == 'preset' and directive['value'] == 'blank':
+                    self.__skip_rows_by_presets['blank'] = True
+                else:
+                    raise ValueError('Not supported skip rows: %s' % directive)
             else:
                 self.__skip_rows_by_comments.append(str(directive))
 
@@ -141,7 +218,10 @@ class Stream(object):
         self.__sample_size = sample_size
         self.__bytes_sample_size = bytes_sample_size
         self.__ignore_blank_headers = ignore_blank_headers
-        self.__blank_header_indexes = []
+        self.__ignore_listed_headers = ignore_listed_headers
+        self.__ignore_not_listed_headers = ignore_not_listed_headers
+        self.__multiline_headers_joiner = multiline_headers_joiner
+        self.__ignored_headers_indexes = []
         self.__force_strings = force_strings
         self.__force_parse = force_parse
         self.__post_parse = copy(post_parse)
@@ -156,6 +236,7 @@ class Stream(object):
         self.__loader = None
         self.__parser = None
         self.__row_number = 0
+        self.__stats = None
 
     def __enter__(self):
         if self.closed:
@@ -171,23 +252,37 @@ class Stream(object):
 
     @property
     def closed(self):
-        '''Returns True if the underlying stream is closed, False otherwise.'''
+        """Returns True if the underlying stream is closed, False otherwise.
+
+        # Returns
+            bool: whether closed
+
+        """
         return not self.__parser or self.__parser.closed
 
     def open(self):
-        '''Opens the stream for reading.'''
+        """Opens the stream for reading.
+
+        # Raises:
+            TabulatorException: if an error
+
+        """
+        source = self.__source
         options = copy(self.__options)
 
-        # Get scheme and format
-        detected_scheme, detected_format = helpers.detect_scheme_and_format(self.__source)
-        scheme = self.__scheme or detected_scheme
-        format = self.__format or detected_format
-
-        # Get compression
+        # Get scheme and format if not already given
         compression = None
-        for type in config.SUPPORTED_COMPRESSION:
-            if self.__compression == type or detected_format == type:
-                compression = type
+        if self.__scheme is None or self.__format is None:
+            detected_scheme, detected_format = helpers.detect_scheme_and_format(source)
+            scheme = self.__scheme or detected_scheme
+            format = self.__format or detected_format
+            # Get compression
+            for type in config.SUPPORTED_COMPRESSION:
+                if self.__compression == type or detected_format == type:
+                    compression = type
+        else:
+            scheme = self.__scheme
+            format = self.__format
 
         # Initiate loader
         self.__loader = None
@@ -210,23 +305,29 @@ class Stream(object):
 
         # Zip compression
         if compression == 'zip' and six.PY3:
-            source = self.__loader.load(self.__source, mode='b')
+            source = self.__loader.load(source, mode='b')
             with zipfile.ZipFile(source) as archive:
                 name = archive.namelist()[0]
+                if 'filename' in options.keys():
+                    name = options['filename']
+                    del options['filename']
                 with archive.open(name) as file:
                     source = tempfile.NamedTemporaryFile(suffix='.' + name)
                     for line in file:
                         source.write(line)
                     source.seek(0)
-            self.__source = source
+            # We redefine loader/format/schema after decompression
             self.__loader = StreamLoader(bytes_sample_size=self.__bytes_sample_size)
             format = self.__format or helpers.detect_scheme_and_format(source.name)[1]
             scheme = 'stream'
 
         # Gzip compression
         elif compression == 'gz' and six.PY3:
-            name = self.__source.replace('.gz', '')
-            self.__source = gzip.open(self.__loader.load(self.__source, mode='b'))
+            name = ''
+            if isinstance(source, str):
+                name = source.replace('.gz', '')
+            source = gzip.open(self.__loader.load(source, mode='b'))
+            # We redefine loader/format/schema after decompression
             self.__loader = StreamLoader(bytes_sample_size=self.__bytes_sample_size)
             format = self.__format or helpers.detect_scheme_and_format(name)[1]
             scheme = 'stream'
@@ -236,6 +337,11 @@ class Stream(object):
             message = 'Compression "%s" is not supported for your Python version'
             raise exceptions.TabulatorException(message % compression)
 
+        # Attach stats to the loader
+        if getattr(self.__loader, 'attach_stats', None):
+            self.__stats = {'size': 0, 'hash': ''}
+            getattr(self.__loader, 'attach_stats')(self.__stats)
+
         # Initiate parser
         parser_class = self.__custom_parsers.get(format)
         if parser_class is None:
@@ -255,7 +361,7 @@ class Stream(object):
             warnings.warn(message, UserWarning)
 
         # Open and setup
-        self.__parser.open(self.__source, encoding=self.__encoding)
+        self.__parser.open(source, encoding=self.__encoding)
         self.__extract_sample()
         self.__extract_headers()
         if not self.__allow_html:
@@ -269,13 +375,16 @@ class Stream(object):
         return self
 
     def close(self):
-        '''Closes the stream.'''
+        """Closes the stream.
+        """
         self.__parser.close()
         self.__row_number = 0
 
     def reset(self):
-        '''Resets the stream pointer to the beginning of the file.'''
+        """Resets the stream pointer to the beginning of the file.
+        """
         if self.__row_number > self.__sample_size:
+            self.__stats = {'size': 0, 'hash': ''}
             self.__parser.reset()
             self.__extract_sample()
             self.__extract_headers()
@@ -283,27 +392,99 @@ class Stream(object):
 
     @property
     def headers(self):
+        """Headers
+
+        # Returns
+            str[]/None: headers if available
+
+        """
         return self.__headers
 
+    @headers.setter
+    def headers(self, headers):
+        """Set headers
+
+        # Arguments
+            str[]: headers
+
+        """
+        self.__headers = headers
+
     @property
     def scheme(self):
+        """Path's scheme
+
+        # Returns
+            str: scheme
+
+        """
         return self.__actual_scheme
 
+    @property
+    def fragment(self):
+        """Path's fragment
+
+        # Returns
+            str: fragment
+
+        """
+        if self.__parser:
+            return getattr(self.__parser, 'fragment', None)
+        return None
+
     @property
     def format(self):
+        """Path's format
+
+        # Returns
+            str: format
+
+        """
         return self.__actual_format
 
     @property
     def encoding(self):
+        """Stream's encoding
+
+        # Returns
+            str: encoding
+
+        """
         return self.__actual_encoding
 
+    @property
+    def size(self):
+        """Returns the BYTE count of the read chunks if available
+
+        # Returns
+            int/None: BYTE count
+
+        """
+        if self.__stats:
+            return self.__stats['size']
+
+    @property
+    def hash(self):
+        """Returns the SHA256 hash of the read chunks if available
+
+        # Returns
+            str/None: SHA256 hash
+
+        """
+        if self.__stats:
+            return self.__stats['hash']
+
     @property
     def sample(self):
-        '''Returns the stream's rows used as sample.
+        """Returns the stream's rows used as sample.
 
         These sample rows are used internally to infer characteristics of the
         source file (e.g. encoding, headers, ...).
-        '''
+
+        # Returns
+            list[]: sample
+
+        """
         sample = []
         iterator = iter(self.__sample_extended_rows)
         iterator = self.__apply_processors(iterator)
@@ -312,30 +493,33 @@ class Stream(object):
         return sample
 
     def iter(self, keyed=False, extended=False):
-        '''Iterate over the rows.
+        """Iterate over the rows.
 
         Each row is returned in a format that depends on the arguments `keyed`
         and `extended`. By default, each row is returned as list of their
         values.
 
-        Args:
-            keyed (bool, optional): When True, each returned row will be a
+        # Arguments
+            keyed (bool, optional):
+                When True, each returned row will be a
                 `dict` mapping the header name to its value in the current row.
-                For example, `[{'name': 'J Smith', 'value': '10'}]`. Ignored if
+                For example, `[{'name'\\: 'J Smith', 'value'\\: '10'}]`. Ignored if
                 ``extended`` is True. Defaults to False.
-            extended (bool, optional): When True, returns each row as a tuple
+            extended (bool, optional):
+                When True, returns each row as a tuple
                 with row number (starts at 1), list of headers, and list of row
                 values. For example, `(1, ['name', 'value'], ['J Smith', '10'])`.
                 Defaults to False.
 
-        Returns:
+        # Raises
+            exceptions.TabulatorException: If the stream is closed.
+
+        # Returns
             Iterator[Union[List[Any], Dict[str, Any], Tuple[int, List[str], List[Any]]]]:
                 The row itself. The format depends on the values of `keyed` and
                 `extended` arguments.
 
-        Raises:
-            exceptions.TabulatorException: If the stream is closed.
-        '''
+        """
 
         # Error if closed
         if self.closed:
@@ -350,8 +534,6 @@ class Stream(object):
 
         # Yield rows from iterator
         for row_number, headers, row in iterator:
-            if self.__force_strings:
-                row = list(map(helpers.stringify_value, row))
             if row_number > self.__row_number:
                 self.__row_number = row_number
                 if extended:
@@ -362,19 +544,19 @@ class Stream(object):
                     yield row
 
     def read(self, keyed=False, extended=False, limit=None):
-        '''Returns a list of rows.
+        """Returns a list of rows.
 
-        Args:
+        # Arguments
             keyed (bool, optional): See :func:`Stream.iter`.
             extended (bool, optional): See :func:`Stream.iter`.
-            limit (int, optional): Number of rows to return. If None, returns
-            all rows. Defaults to None.
+            limit (int, optional):
+                Number of rows to return. If None, returns all rows. Defaults to None.
 
-        Returns:
+        # Returns
             List[Union[List[Any], Dict[str, Any], Tuple[int, List[str], List[Any]]]]:
                 The list of rows. The format depends on the values of `keyed`
                 and `extended` arguments.
-        '''
+        """
         result = []
         rows = self.iter(keyed=keyed, extended=extended)
         for count, row in enumerate(rows, start=1):
@@ -384,16 +566,18 @@ class Stream(object):
         return result
 
     def save(self, target, format=None,  encoding=None, **options):
-        '''Save stream to the local filesystem.
+        """Save stream to the local filesystem.
 
-        Args:
+        # Arguments:
             target (str): Path where to save the stream.
-            format (str, optional): The format the stream will be saved as. If
+            format (str, optional):
+                The format the stream will be saved as. If
                 None, detects from the ``target`` path. Defaults to None.
-            encoding (str, optional): Saved file encoding. Defaults to
-                ``config.DEFAULT_ENCODING``.
+            encoding (str, optional):
+                Saved file encoding. Defaults to ``config.DEFAULT_ENCODING``.
             **options: Extra options passed to the writer.
-        '''
+
+        """
 
         # Get encoding/format
         if encoding is None:
@@ -433,6 +617,10 @@ class Stream(object):
         for _ in range(self.__sample_size):
             try:
                 row_number, headers, row = next(self.__parser.extended_rows)
+                if self.__headers_row and self.__headers_row >= row_number:
+                    if self.__check_if_row_for_skipping(row_number, headers, row):
+                        self.__headers_row += 1
+                        self.__headers_row_last += 1
                 self.__sample_extended_rows.append((row_number, headers, row))
             except StopIteration:
                 break
@@ -464,21 +652,39 @@ class Stream(object):
                     if len(headers) > index and headers[index] is not None:
                         if not self.__headers[index]:
                             self.__headers[index] = headers[index]
-                        elif not self.__headers[index].endswith(headers[index]):
-                            self.__headers[index] += ' ' + headers[index]
+                        else:
+                            # https://github.com/frictionlessdata/tabulator-py/issues/292
+                            if (not self.__options.get('fill_merged_cells') or
+                                    not self.__headers[index].endswith(headers[index])):
+                                self.__headers[index] += self.__multiline_headers_joiner + headers[index]
             if row_number == self.__headers_row_last:
                 break
 
-        # Ignore blank headers
-        if self.__ignore_blank_headers:
-            self.__blank_header_indexes = []
+        # Ignore headers
+        if (self.__ignore_blank_headers or
+                self.__ignore_listed_headers is not None or
+                self.__ignore_not_listed_headers is not None):
+            self.__ignored_headers_indexes = []
             raw_headers, self.__headers = self.__headers, []
             for index, header in list(enumerate(raw_headers)):
+                ignore = False
+                # Ignore blank headers
                 if header in ['', None]:
-                    self.__blank_header_indexes.append(index)
+                    ignore = True
+                # Ignore listed headers
+                if self.__ignore_listed_headers is not None:
+                    if header in self.__ignore_listed_headers:
+                        ignore = True
+                # Ignore not-listed headers
+                if self.__ignore_not_listed_headers is not None:
+                    if header not in self.__ignore_not_listed_headers:
+                        ignore = True
+                # Add to the list and skip
+                if ignore:
+                    self.__ignored_headers_indexes.append(index)
                     continue
                 self.__headers.append(header)
-            self.__blank_header_indexes = sorted(self.__blank_header_indexes, reverse=True)
+            self.__ignored_headers_indexes = sorted(self.__ignored_headers_indexes, reverse=True)
 
         # Remove headers from data
         if not keyed_source:
@@ -512,18 +718,13 @@ class Stream(object):
                         row = [keyed_row.get(header) for header in self.__headers]
                     headers = self.__headers
 
-                # Skip row by numbers
-                if row_number in self.__skip_rows_by_numbers:
+                # Skip rows by numbers/comments
+                if self.__check_if_row_for_skipping(row_number, headers, row):
                     continue
 
-                # Skip row by comments
-                match = lambda comment: row[0].startswith(comment)
-                if list(filter(match, self.__skip_rows_by_comments)):
-                    continue
-
-                # Ignore blank headers
-                if self.__blank_header_indexes:
-                    for index in self.__blank_header_indexes:
+                # Ignore headers
+                if self.__ignored_headers_indexes:
+                    for index in self.__ignored_headers_indexes:
                         if index < len(row):
                             row = row[:index] + row[index+1:]
 
@@ -553,6 +754,12 @@ class Stream(object):
                 if i - n not in rows_to_skip:
                     yield row
 
+        # Force values to strings processor
+        def force_strings_processor(extended_rows):
+            for row_number, headers, row in extended_rows:
+                row = list(map(helpers.stringify_value, row))
+                yield (row_number, headers, row)
+
         # Form a processors list
         processors = [builtin_processor]
         # if we have to delete some rows with negative index (counting from the end)
@@ -560,9 +767,41 @@ class Stream(object):
             processors.insert(0, skip_negative_rows)
         if self.__post_parse:
             processors += self.__post_parse
+        if self.__force_strings:
+            processors.append(force_strings_processor)
 
         # Apply processors to iterator
         for processor in processors:
             iterator = processor(iterator)
 
         return iterator
+
+    def __check_if_row_for_skipping(self, row_number, headers, row):
+
+        # Skip by number
+        if row_number in self.__skip_rows_by_numbers:
+            return True
+
+        # Get first cell
+        cell = row[0] if row else None
+
+        # Handle blank cell/row
+        if cell in [None, '']:
+            if '' in self.__skip_rows_by_comments:
+                return True
+            if self.__skip_rows_by_presets.get('blank'):
+                if not list(filter(lambda cell: cell not in [None, ''], row)):
+                    return True
+            return False
+
+        # Skip by pattern
+        for pattern in self.__skip_rows_by_patterns:
+            if bool(pattern.match(cell)):
+                return True
+
+        # Skip by comment
+        for comment in filter(None, self.__skip_rows_by_comments):
+            if cell.startswith(comment):
+                return True
+
+        return False
